#!/usr/bin/perl -w
use strict;
use Getopt::Long;

my ($users, $honest, $file, $oneRow, $twoCols) = (5, 4, "model", undef, undef);
GetOptions(
	"file=s"	=> \$file,				# output filename. 2 files, file.nm and file.pctl will be created
	"users=i"	=> \$users,				# total number of users
	"honest=i"	=> \$honest,			# number of honest users
	"one-row"	=> \$oneRow,			# generate model/formulas for 1 user
	"two-cols"	=> \$twoCols,			# generate formulas for only 2 observables
)	or exit;

$users >= $honest	or die "users must be >= honest";

open F, "> $file.nm"	or die "cannot open $file.nm";

# --- Declarations ---
#
print F qq{
nondeterministic

const double pf;

const int honestNo = $honest;
const int userNo = $users;

global initiator: [-1..honestNo] init -1;

global curUser: [-1..userNo] init -1;		// the user who has the message now
global prevUser: [-1..honestNo] init -1;		// the user who *sent* the message last

global delivered: bool init false;		// has the message arrived to the server?
global detected: [-1..honestNo] init -1;		// the user detected by the attacker

};

# --- Init ---
#
print F q{
module Init
	// select an initiator
};
for my $i (0..($oneRow ? 0 : $honest-1)) {
	print F "\t[] initiator = -1 & curUser = -1 -> (initiator' = $i);\n";
}
print F q{
	// forward the message from the initiator to a user
	[] initiator != -1 & curUser = -1 ->
};
for my $i (0..$users-1) {
	print F "\t\t1/userNo: (prevUser' = initiator) & (curUser' = $i)" . ($i < $users-1 ? " +" : ";") . "\n";
}
print F q{
endmodule

};

# --- Users ---
#
print F q{
module User
	[] curUser >= 0 & curUser < honestNo & !delivered ->
			(1-pf/100): (delivered' = true) +
};
for my $i (0..$users-1) {
	print F "\t\t(pf/100)/userNo: (prevUser' = curUser) & (curUser' = $i)" .
			($i < $users-1 ? " +" : ";") . "\n";
}
print F q{
endmodule

};

# --- Atacker ---
#
print F q{
module Attacker
	[] curUser >= honestNo & !delivered ->
		(delivered' = true) &
		(detected' = prevUser);
endmodule
};

close F;

####################   PCTL  ##############################

open F, "> $file.pctl"	or die "cannot open $file.pctl";

# --- Observables ---
#
print F "label \"noobs\" = delivered & detected = -1;\n\n";

for my $i (0..$honest-1) {
	print F "label \"observ$i\" = delivered & detected = $i;\n";
}
print F "\n";

# --- Users ---
#
for my $i (0..($oneRow ? 0 : $honest-1)) {
	print F "label \"user$i\" = initiator = $i & curUser = -1;\n";
}
print F "\n";

# --- Conditional probabilities
#

for my $i (0..($oneRow ? 0 : $honest-1)) {
	print F "Pmax=? [ true U \"noobs\" { \"user$i\" } ]\n";

	my $col = $i == 0 ? 1: 0	if $twoCols;

	for my $o (0..$honest-1) {
		next if $twoCols && !($o == $i || $o == $col);

		print F "Pmax=? [ true U \"observ$o\" { \"user$i\" } ]\n";
	}
	print F "\n";
}


